/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

"use strict";

const base64 = require("sdk/base64");
const { Ci } = require("chrome");
const { getTabForChannel, getUniqueTabId } = require("./lib-tabutils");

class Connection {
  constructor(event) {
    this.valid = true;
    let httpChannel = event.subject.QueryInterface(Ci.nsIHttpChannel);

    let targetUri = httpChannel.URI;
    if (targetUri.scheme !== 'https' || !(httpChannel.loadFlags & Ci.nsIChannel.LOAD_INITIAL_DOCUMENT_URI)) {
      this.valid = false;
      return;
    }

    // Get the browser tab, related to the current connection
    let tab = null;
    try {
      tab = getTabForChannel(httpChannel);
    } catch (e) {
      console.debug('Error: No tab for connection ' + targetUri.spec);
      tab = null;
    }

    // Do not continue if no tab is associated (e.g. OCSP responses)
    if (!tab) {
      this.valid = false;
      return;
    }

    this.targetDomain = targetUri.host;
    this.tabId = getUniqueTabId(tab);

    let secInfo = httpChannel.securityInfo;
    if (secInfo && secInfo instanceof Ci.nsISSLStatusProvider) {
      let sslStatus = secInfo.QueryInterface(Ci.nsISSLStatusProvider).SSLStatus.QueryInterface(Ci.nsISSLStatus);
      if (sslStatus) {
        let derBytes = sslStatus.serverCert.getRawDER({});
        let derStr = '';
        for (let i = 0; i < derBytes.length; i++) {
          derStr = derStr + String.fromCharCode(derBytes[i]);
        }
        let derB64 = base64.encode(derStr);
        this.pemCert = derB64.replace(/(\S{64}(?!$))/g, "$1\r\n");
      }
    }
  }
}

exports.Connection = Connection;