/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

"use strict";

const XMLHttpRequest = require("sdk/net/xhr").XMLHttpRequest;

const serviceUri = 'https://www.buergerkarte.at/test-signature-verification/services/VerificationService.VerificationServicePortSOAP/';
const postData = '<?xml version="1.0" encoding="UTF-8"?>' +
    '<env:Envelope xmlns:env="http://schemas.xmlsoap.org/soap/envelope/" env:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">' +
    '<env:Body>' +
      '<VerifyDocumentRequest xmlns="http://reference.e-government.gv.at/namespace/verificationservice/20120922#">' +
        '<Version>2</Version>' +
        '<Document>_CERTPEM_</Document>' +
        '<FileId>cert.cer</FileId>' +
        '<RequestPDFReport>false</RequestPDFReport>' +
        '<RequestSignerCertifcate>false</RequestSignerCertifcate>' +
        '<Language>en</Language>' +
      '</VerifyDocumentRequest>' +
    '</env:Body>' +
    '</env:Envelope>';

let trustCache = {};

function requestStatus(tabId, domain, certPem, callback) {
  let cacheResult = checkCache(domain);
  if (cacheResult) {
    //console.log('CACHE callback!');
    callback(tabId, domain, cacheResult.isTrusted);
    return;
  }

  let req = new XMLHttpRequest();
  req.onload = event => {
    //console.log('RESPONSE: ' + domain + ' | ' + event.target.responseText);

    let responseXml = event.target.responseXML;
    let nsPrefix = (responseXml.getElementsByTagName('ns2:SignatureCheck').length > 0) ? 'ns2:' : '';
    let isTrusted = false;
    try {
      let c = responseXml.
      getElementsByTagName(nsPrefix + 'CertificateCheck')[0].
      getElementsByTagName(nsPrefix + 'Code')[0].textContent;
      if (c === '0') isTrusted = true;
    } catch (e) {
      console.log('CertificateCheck Code not found? ' + e);
    }

    addCacheResult(domain, isTrusted, event.target.responseText);
    callback(tabId, domain, isTrusted);
  };
  req.onerror = e => {
    console.log('requestStatus error! ' + e);
    callback(tabId, domain, false);
  };

  try {
    req.open("POST", serviceUri);
    req.setRequestHeader("Content-Type", "text/xml;charset=UTF-8");
    req.setRequestHeader("SOAPAction", "http://localhost:8080/signature-verification/services/verifyDocument");
    req.send(postData.replace('_CERTPEM_', certPem));
    //console.log("REQUEST: \n" + postData.replace('_CERTPEM_', certPem));
  } catch (e) {
    console.log('requestStatus exception! ' + e);
    callback(tabId, domain, false);
  }
}

function addCacheResult(domain, trusted, trustDetails) {
  trustCache[domain] = { isTrusted: trusted, details: trustDetails, entryTime: Date.now() };
}

function checkCache(domain) {
  if (trustCache[domain] && Date.now() < (trustCache[domain].entryTime + 60*60*1000)) {
    //console.log('CACHE HIT! ' + trustCache[domain].entryTime + ' | ' + (Date.now() - 60*60*1000));
    return trustCache[domain];
  }

  return (trustCache[domain] ? trustCache[domain] : null);
}

exports.requestStatus = requestStatus;
exports.checkCache = checkCache;