/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

"use strict";

const ui = require("./ui");
const trustService = require("./lib-trust");
const tabs = require("sdk/tabs");
const events = require("sdk/system/events");
const { Connection } = require("./connection");
const { URL } = require("sdk/url");

let openTabs = {};
events.on("http-on-examine-response", collectConnection, true);
events.on("http-on-examine-cached-response", collectConnection, true);
events.on("http-on-examine-merged-response", collectConnection, true);

function collectConnection(event) {
  let connection = new Connection(event);
  if (connection.valid) {
    openTabs[connection.tabId] = {
      domain: connection.targetDomain,
      connection: connection
    };
  }
}

function trustCallback(tabId, domain, isTrusted) {
  if (!openTabs[tabId]) return; // Tab could have been closed meanwhile
  if (openTabs[tabId].domain != domain) return; // Domain loaded in tab has changed meanwhile?

  ui.setIcon(tabId, isTrusted, true);
}

function onReady(tab) {
  let url = URL(tab.url);
  let domain = url.host;
  if (!domain || url.scheme !== 'https' || !openTabs[tab.id]) {
    ui.setIcon(tab.id, false, false);
    return;
  }

  trustService.requestStatus(tab.id, domain, openTabs[tab.id].connection.pemCert, trustCallback);

  if (ui.isShowingPanel()) {
    ui.showPanel();
  }
}

tabs.on('open', () => ui.hidePanel());
tabs.on('pageshow', onReady); // Event pageshow: like load, ready but bfcache retrieval
tabs.on('activate', onReady); // Event activate: Tab activation
tabs.on('close', tab => delete openTabs[tab.id]);

exports.getOpenTab = () => openTabs[tabs.activeTab.id];
