/*
 * Copyright 2017 by A-SIT, Secure Information Technology Center Austria
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

"use strict";

const tabs = require("sdk/tabs");
const data = require("sdk/self").data;
const { UrlbarButton } = require("./lib-urlbarbutton");
const trustService = require("./lib-trust");
const main = require("./main");

const panel = require("sdk/panel").Panel({
  width: 535,
  height: 460,
  name : "A-SIT-Panel",
  contextMenu: true,
  contentURL: data.url('popup.html'),
  contentScriptFile: [
    data.url("libs/jquery-3.1.1.min.js"),
    data.url("libs/bootstrap-3.3.5.min.js"),
    data.url("libs/pkijs/asn1.js"),
    data.url("libs/pkijs/common.js"),
    data.url("libs/pkijs/x509_schema.js"),
    data.url("libs/pkijs/x509_simpl.js"),
    data.url("popup.js")
  ],
  contentScriptWhen: 'ready',
  onShow: showPanel,
  onMessage: message => {
    // Hide the panel when receiving a close message
    if (message == 'close') {
      panel.hide();
    }
  }
});

const asitbutton = UrlbarButton({
  id : 'asit-panel',
  image : data.url('./icons/flag_white_eps_crossed_16.png'),
  collapsed: true,
  onClick: (tabId, event, button) => {
    panel.show(button);
  }
});

function showPanel() {
  let openTab = main.getOpenTab();

  // Nothing to show if no connections are known for the current tab
  if (!openTab) {
    panel.port.emit('connData', null, null);
    return;
  }

  let cacheResults = trustService.checkCache(openTab.domain);
  if (!cacheResults) {
    console.log('No cache entry for ' + openTab.domain + '?');
    return;
  }
  let responseXml = cacheResults.details;
  panel.port.emit('connData', responseXml, openTab.connection.pemCert);
}

function setIcon(tabId, isTrusted, visible) {
  if (!visible) {
    asitbutton.setVisibility(false, tabId);
    return;
  }

  let image = (isTrusted ? './icons/flag_white_eps_16.png' : './icons/flag_white_eps_crossed_16.png');
  asitbutton.setImage(data.url(image), tabId);
  asitbutton.setVisibility(true, tabId);
}

require('sdk/system/unload').when(function shutdown(reason) {
  if (reason !== 'shutdown') {
    asitbutton.remove();
  }
});

exports.setIcon = setIcon;
exports.showPanel = showPanel;
exports.hidePanel = () => panel.hide();
exports.isShowingPanel = () => panel.isShowing;